/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.jsf.conventions.bean;

import com.jsf.conventions.event.ModalInitialization;
import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.PostConstruct;
import com.jsf.conventions.bean.state.CrudState;
import com.jsf.conventions.bean.state.State;
import com.jsf.conventions.model.ConventionsDataModel;
import com.jsf.conventions.qualifier.PersistentClass;
import com.jsf.conventions.qualifier.Service;
import com.jsf.conventions.service.BaseService;
import com.jsf.conventions.util.AnnotationUtils;
import com.jsf.conventions.util.BeanManagerController;
import com.jsf.conventions.util.MessagesController;
import com.jsf.conventions.util.ResourceBundle;
import java.lang.annotation.Annotation;
import javax.enterprise.event.Event;
import javax.inject.Inject;
import org.primefaces.model.LazyDataModel;
import org.primefaces.model.SortOrder;

/**
 * Base implementation of managedBeans
 *
 * @author Rafael M. Pestano Mar 17, 2011 10:18:44 PM
 */
public abstract class BaseMBean<T> implements Serializable {

    private T entity;
    private T entityToRemove;
    private BaseService baseService;
    private State beanState;
    private LazyDataModel<T> dataModel;
    private int rowCount;
    private Map<String, Object> filter = new HashMap<String, Object>();
    private String createMessage;
    private String deleteMessage;
    private String updateMessage;
    @Inject
    private Event<ModalInitialization> modalInitEvent;
    private ResourceBundle bundle;

    // getter & setters
    public BaseService getBaseService() {
        return baseService;
    }

    //override in managedBean
    public void setBaseService(BaseService baseService) {
        this.baseService = baseService;
    }

    public T getEntity() {
        return entity;
    }

    public void setEntity(T entity) {
        this.entity = entity;
    }

    public T getEntityToRemove() {
        return entityToRemove;
    }

    public void setEntityToRemove(T entityToRemove) {
        this.entityToRemove = entityToRemove;
    }

    public State getBeanState() {
        return beanState;
    }

    public void setBeanState(State beanState) {
        this.beanState = beanState;
    }

    public LazyDataModel<T> getDataModel() {
        return dataModel;
    }

    public void setDataModel(LazyDataModel<T> dataModel) {
        this.dataModel = dataModel;
    }

    public int getRowCount() {
        return rowCount;
    }

    public void setRowCount(int rowCount) {
        this.rowCount = rowCount;
    }

    public ResourceBundle getResourceBundle() {
        return bundle;
    }

    @Inject
    public void setResourceBundle(ResourceBundle resourceBundle) {
        this.bundle = resourceBundle;
    }

    public String getCreateMessage() {
        return createMessage;
    }

    public void setCreateMessage(String createMessage) {
        this.createMessage = createMessage;
    }

    public String getDeleteMessage() {
        return deleteMessage;
    }

    public void setDeleteMessage(String deleteMessage) {
        this.deleteMessage = deleteMessage;
    }

    public Map<String, Object> getFilter() {
        return filter;
    }

    public void setFilter(Map<String, Object> filter) {
        this.filter = filter;
    }

    public String getUpdateMessage() {
        return updateMessage;
    }

    public void setUpdateMessage(String updateMessage) {
        this.updateMessage = updateMessage;
    }

    @PostConstruct
    public void init() {
        setEntity(this.create());
        setCreateMessage("Record created successfully");
        setUpdateMessage("Record updated successfully");
        setDeleteMessage("Record deleted successfully");
        if (serviceInitialized()) { //baseService must be set before calling init
            dataModel = new LazyDataModel<T>() {
                @Override
                public List<T> load(int first, int pageSize, String sortField, SortOrder sortOrder, Map<String, String> dataTableFilters) {
                    ConventionsDataModel<T> wrappedData;
                    wrappedData = baseService.configFindPaginated(first, pageSize, sortField, sortOrder, dataTableFilters, filter);
                    rowCount = wrappedData.getRowCount();
                    this.setRowCount(rowCount);//datatable rowCount
                    return wrappedData.getData();
                }
            };
        }
    }

    public T create() {
        try {
            for (Annotation annotation : getClass().getAnnotations()) {
                if (annotation instanceof PersistentClass) {
                    PersistentClass p = (PersistentClass) annotation;
                    Class c = p.value();
                    return (T) c.newInstance();
                }
            }//if no annotation found then try to create via reflection
            return ((Class<T>) ((ParameterizedType) getClass().getGenericSuperclass()).getActualTypeArguments()[0]).newInstance();
        } catch (Exception ex) {
            Logger.getLogger(BaseMBean.class.getName()).log(Level.WARNING, null, ex);
        }
        return null;
    }

    public boolean isInsertState() {
        return CrudState.INSERT.equals(this.beanState);
    }

    public boolean isUpdateState() {
        return CrudState.UPDATE.equals(this.beanState);
    }

    public boolean isFindState() {
        return CrudState.FIND.equals(this.beanState);
    }

    //actions
    public void store() {
        if (isInsertState()) {
            MessagesController.addInfo(getCreateMessage());
        } else {
            MessagesController.addInfo(getUpdateMessage());
        }
        baseService.store(entity);
        setBeanState(CrudState.UPDATE);
    }

    public void delete() {
        this.setEntity(entityToRemove);
        this.remove(getEntity());
        MessagesController.addInfo(deleteMessage);
    }

    public void remove(T entity) {
        baseService.remove(entity);
    }

    /**
     * called by editButton
     */
    public String prepareUpdate() {
        setBeanState(CrudState.UPDATE);
        return afterPrepareUpdate();
    }

    /**
     * called by addButton
     */
    public String prepareInsert() {
        setBeanState(CrudState.INSERT);
        setEntity(this.create());
        return afterPrepareInsert();
    }

    /**
     * @return view id default is the calling page
     */
    public String afterPrepareInsert() {
        return null;
    }

    /**
     * @return view id default is the calling page
     */
    public String afterPrepareUpdate() {
        return null;
    }

    /**
     *
     * called on framework:filterButton action
     */
    public void find() {
    }

    /**
     * called on framework:removeButton with persistentRemove = false action
     *
     */
    public void removeFromList() {
    }

    /**
     * reset entity state
     */
    public void resetEntity() {
        if (isUpdateState()) {
            setBeanState(CrudState.INSERT);
        }
        this.entity = this.create();
        clearMBean();
    }

    /**
     * method responsible for clearing fields referenced in the managed bean and
     * not in the entity
     *
     */
    public void clearMBean() {
    }

    /**
     * used to pass parameters to a modal
     *
     * @param modalName name of the modal which must receive the parameters
     * @param parameters parameters to send
     */
    public void initModal(String modalName, Map<String, Object> parameters) {
        modalInitEvent.fire(new ModalInitialization(parameters, modalName));
    }

    public void setFindState() {
        setBeanState(CrudState.FIND);
    }

    public void setInsertState() {
        setBeanState(CrudState.INSERT);
    }

    public void setUpdateState() {
        setBeanState(CrudState.UPDATE);
    }

    private boolean serviceInitialized() {
        if (getBaseService() != null) {
            return true;
        }
        Service service = AnnotationUtils.findServiceAnnotation(getClass());
        if (service != null && !"".equals(service.name())) {
            try{
                this.setBaseService((BaseService)BeanManagerController.getBeanByName(service.name()));
                return true;
            }catch (Exception ex){
                ex.printStackTrace();
                return false;
            }
        }
        return false;
    }

}