/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.jsf.conventions.service.impl;

import com.jsf.conventions.dao.HibernateDao;
import com.jsf.conventions.exception.BusinessException;
import java.io.Serializable;
import java.util.List;
import java.util.Map;
import com.jsf.conventions.model.ConventionsDataModel;
import com.jsf.conventions.service.BaseService;
import javax.persistence.EntityManager;
import org.apache.myfaces.extensions.cdi.jpa.api.Transactional;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.loader.custom.ScalarReturn;
import org.hibernate.transform.ResultTransformer;
import org.primefaces.model.SortOrder;

/**
 * Non EJB dependent service
 * @author Rafael M. Pestano Mar 19, 2011 11:55:37 AM
 */

public abstract class BaseCustomServiceImpl<T, K extends Serializable> implements BaseService<T, K>, Serializable {

    
    protected HibernateDao dao;
    
     @Override
    public void store(T entity){
         try{
            doStore(entity);
        }catch(BusinessException be){
            throw be;
        }
        
    }
    @Transactional
    public void doStore(T entity){
        this.beforeStore(entity);
        this.saveOrUpdate(entity);
        this.afterStore(entity);
    }
    @Override
     public void afterStore(T entity) {
    }
    
    @Override
    public void beforeStore(T entity) {
    }
    
    @Override
    @Transactional
    public void remove(T entity){
        this.doRemove(entity);
    }

    @Override
    public void beforeRemove(T entity) {
    }
    
    @Transactional
    private void doRemove(T entity) {
        this.beforeRemove(entity);
        this.delete(entity);
        this.afterRemove(entity);
    }

    @Override
    public void afterRemove(T entity) {
    }

    @Override
    public ConventionsDataModel<T> configFindPaginated(int first, int pageSize, String sortField, SortOrder sortOrder, Map<String, String> filters, Map<String, Object> externalFilter) {
        return getDao().configFindPaginated(first, pageSize, sortField, sortOrder, filters, externalFilter);
    }

    @Override
    public int countAll() {
        return getDao().countAll();
    }
    
    @Override
    public T load(K id) {
        return (T) getDao().load(id);
    }

    @Override
    public T get(K id) {
        return (T) getDao().get(id);
    }


    @Override
    public final void save(T entity) {
        getDao().save(entity);
    }

    @Override
    public final void update(T entity) {
        getDao().update(entity);
    }

    @Override
    public final void delete(T entity) {
        getDao().delete(entity);
        flushSession();
    }

    @Override
    public T refresh(T entity) {
        return (T) getDao().refresh(entity);
    }

    @Override
    public final void saveOrUpdate(T entity) {
        getDao().saveOrUpdate(entity);
        flushSession();
    }

    @Override
    public List<T> findAll() {
        return getDao().findAll();
    }

    @Override
    public List<T> findAll(Integer first, Integer max) {
        return getDao().findAll(first, max);
    }

    @Override
    public void setPersistentClass(Class<T> persistentClass) {
       getDao().setPersistentClass(persistentClass);
    }
    
    @Override
    public Class<T> getPersistentClass(){
        return getDao().getPersistentClass();
    }

    
    public HibernateDao getDao() {
        return dao;
    }

    public void setDao(HibernateDao dao) {
        this.dao = dao;
    }
    
     @Override
    public Session getSession() {
        return dao.getSession();
    }

    @Override
    public List<T> findByExample(T entity) {
        return dao.findByExample(entity);
    }
    
    @Override
    public List<T> findByExample(final T entity, MatchMode matchMode) {
        return dao.findByExample(entity,matchMode);
    }
    
    @Override
    public List<T> findByExample(final T entity,int maxResult) {
        return dao.findByExample(entity,maxResult);
    }
    
    @Override
    public List<T> findByExample(final T entity,int maxResult, MatchMode matchMode) {
        return dao.findByExample(entity,maxResult,matchMode);
    }

    @Override
    public T findOneByExample(T entity) {
        return (T) dao.findOneByExample(entity);
    }
    
    @Override
    public List<T> findByExample(T exampleInstance, String[] excludeProperty) {
       return dao.findByExample(exampleInstance, excludeProperty);
    }

    @Override
    public T findOneByExample(T entity, MatchMode matchMode) {
       return (T) dao.findOneByExample(entity, matchMode);
    }

    @Override
    public List<T> findByCriteria(DetachedCriteria criteriaObject, int first, int maxResult) {
        return dao.findByCriteria(criteriaObject, first, maxResult);
    }

    @Override
    public List<T> findByCriteria(DetachedCriteria criteriaObject) {
        return dao.findByCriteria(criteriaObject);
    }

    @Override
    public ConventionsDataModel<T> findPaginated(int first, int pageSize, String sortField, SortOrder sortOrder, DetachedCriteria dc) {
        return dao.findPaginated(first, pageSize, sortField, sortOrder, dc);
    }

    @Override
    public Long getRowCount(DetachedCriteria criteria) {
        return dao.getRowCount(criteria);
    }
    
    @Override
    public DetachedCriteria getDetachedCriteria(){
        return DetachedCriteria.forClass(getPersistentClass());
    }
    
    @Override
    public Criteria getCriteria() {
       return dao.getCriteria();
    }
    
     /**
     * 
     * @param nativeQuery
     * @param params
     * @param class entity
     * @param result transformer
     * @param Scalar
     * @return 
     */
    @Override
    public List findByNativeQuery(String nativeQuery,Map params,Class entity,ResultTransformer rt,ScalarReturn scalar){
        return dao.findByNativeQuery(nativeQuery, params, entity, rt, scalar);
    }
    

    public void flushSession(){
        Session session = getSession();
        if(session.isOpen() && session.isDirty()){
            session.flush();
        }
    }
    
    @Override
    public EntityManager getEntityManager(){
        return dao.getEntityManager();
    }
}
